Encoding: UTF-8
Package: optparse
Type: Package
Title: Command Line Option Parser
Version: 1.8.2
Authors@R: c(person("Trevor L.", "Davis", role=c("aut", "cre"),
             email="trevor.l.davis@gmail.com",
             comment = c(ORCID = "0000-0001-6341-4639")),
    person("Allen", "Day", role="aut", comment="Code and documentation ported from the getopt package."),
    person("Python Software Foundation", role="ctb", comment="Some documentation from the optparse Python module."),
    person("Steve", "Lianoglou", role="ctb"),
    person("Jim", "Nikelski", role="ctb"),
    person("Kirill", "Müller", role="ctb"),
    person("Peter", "Humburg", role="ctb"),
    person("Rich", "FitzJohn", role="ctb"),
    person("Gyu Jin", "Choi", role="ctb"))
Description: A command line parser inspired by Python's 'optparse' library to
    be used with Rscript to write "#!" shebang scripts that accept short and
    long flag/options.
License: GPL (>= 2)
Copyright: See file (inst/)COPYRIGHTS.
URL: https://github.com/trevorld/r-optparse,
        https://trevorldavis.com/R/optparse/
BugReports: https://github.com/trevorld/r-optparse/issues
LazyLoad: yes
Depends: R (>= 3.6.0)
Imports: methods
Suggests: knitr (>= 1.15.19), rmarkdown, stringr, testthat
Config/testthat/edition: 3
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-15 16:45:06 UTC; trevorld
Author: Trevor L. Davis [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6341-4639>),
  Allen Day [aut] (Code and documentation ported from the getopt
    package.),
  Python Software Foundation [ctb] (Some documentation from the optparse
    Python module.),
  Steve Lianoglou [ctb],
  Jim Nikelski [ctb],
  Kirill Müller [ctb],
  Peter Humburg [ctb],
  Rich FitzJohn [ctb],
  Gyu Jin Choi [ctb]
Maintainer: Trevor L. Davis <trevor.l.davis@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-17 07:10:02 UTC
Built: R 4.5.2; ; 2026-04-20 23:21:40 UTC; unix
