Encoding: UTF-8
Package: optparse
Type: Package
Title: Command Line Option Parser
Version: 1.7.5
Authors@R: c(person("Trevor L.", "Davis", role=c("aut", "cre"),
             email="trevor.l.davis@gmail.com",
             comment = c(ORCID = "0000-0001-6341-4639")),
    person("Allen", "Day", role="ctb", comment="Some documentation and examples ported from the getopt package."),
    person("Python Software Foundation", role="ctb", comment="Some documentation from the optparse Python module."),
    person("Steve", "Lianoglou", role="ctb"),
    person("Jim", "Nikelski", role="ctb"),
    person("Kirill", "Müller", role="ctb"),
    person("Peter", "Humburg", role="ctb"), 
    person("Rich", "FitzJohn", role="ctb"),
    person("Gyu Jin", "Choi", role="ctb"))
Description: A command line parser inspired by Python's 'optparse' library to
    be used with Rscript to write "#!" shebang scripts that accept short and
    long flag/options.
License: GPL (>= 2)
Copyright: See file (inst/)COPYRIGHTS.
URL: https://github.com/trevorld/r-optparse
BugReports: https://github.com/trevorld/r-optparse/issues
LazyLoad: yes
Depends: R (>= 3.6.0)
Imports: methods, getopt (>= 1.20.2)
Suggests: knitr (>= 1.15.19), stringr, testthat
VignetteBuilder: knitr
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2024-04-16 00:25:36 UTC; trevorld
Author: Trevor L. Davis [aut, cre] (<https://orcid.org/0000-0001-6341-4639>),
  Allen Day [ctb] (Some documentation and examples ported from the getopt
    package.),
  Python Software Foundation [ctb] (Some documentation from the optparse
    Python module.),
  Steve Lianoglou [ctb],
  Jim Nikelski [ctb],
  Kirill Müller [ctb],
  Peter Humburg [ctb],
  Rich FitzJohn [ctb],
  Gyu Jin Choi [ctb]
Maintainer: Trevor L. Davis <trevor.l.davis@gmail.com>
Repository: CRAN
Date/Publication: 2024-04-16 20:00:06 UTC
Built: R 4.4.0; ; 2026-07-18 10:23:41 UTC; unix
