# Generated by roxygen2: do not edit by hand

export(IndentedHelpFormatter)
export(OptionParser)
export(OptionParserOption)
export(TitledHelpFormatter)
export(add_option)
export(make_option)
export(parse_args)
export(parse_args2)
export(print_help)
exportClasses(OptionParser)
exportClasses(OptionParserOption)
import(getopt)
import(methods)
importFrom(utils,tail)
